! calculate fire extent and impact on plants.
! This is treated on a pseudo-daily basis (fireindex has a long-term memory).
! We only take into account the natural litter, as agricultural and
!   natural PFTs are usually spatially separated.
! Fire decreases the biomass per m**2 of natural ground.
! Grasses are totally burned.
! When the vegetation is dynamic, it also decreases the density of individuals.
! Fire burns litter on the ground.
!
! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_stomate/lpj_fire.f90,v 1.3 2002/09/24 21:12:16 ssipsl Exp $
!
MODULE lpj_fire

  ! modules used:

  USE ioipsl
  USE stomate_constants

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC fire,fire_clear

  ! first call
  LOGICAL, SAVE                                                   :: firstcall = .TRUE.

CONTAINS


  SUBROUTINE fire_clear
    firstcall = .TRUE.
  END SUBROUTINE fire_clear

  SUBROUTINE fire (npts, dt, space_nat, litterpart, &
                   litterhum_daily, t2m_daily, lignin_struc, &
                   fireindex, firelitter, biomass, ind, &
                   litter, dead_leaves, bm_to_litter, black_carbon, &
                   co2_fire)

    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                             :: npts
    ! Time step in days
    REAL(stnd), INTENT(in)                                          :: dt
    ! fraction of total space that is natural
    REAL(stnd), DIMENSION(npts), INTENT(in)                         :: space_nat
    ! fraction of litter above the ground belonging to different PFTs
    REAL(stnd), DIMENSION(npts,npft,nlitt), INTENT(in)              :: litterpart
    ! Daily litter moisture (between 0 and 1)
    REAL(stnd), DIMENSION(npts), INTENT(in)                         :: litterhum_daily
    ! Daily 2 meter temperature (K)
    REAL(stnd), DIMENSION(npts), INTENT(in)                         :: t2m_daily
    ! ratio Lignine/Carbon in structural litter, above and below ground,
    !   natural and agricultural (gC/(m**2 of nat/agri ground))
    REAL(stnd), DIMENSION(npts,nvegtypes,nlevs), INTENT(in)         :: lignin_struc

    ! 0.2 modified fields

    ! Probability of fire
    REAL(stnd), DIMENSION(npts,nvegtypes), INTENT(inout)            :: fireindex
    ! Longer term total natural litter above the ground, gC/m**2 of natural ground
    REAL(stnd), DIMENSION(npts,nvegtypes), INTENT(inout)            :: firelitter
    ! biomass (gC/m**2)
    REAL(stnd), DIMENSION(npts,npft,nparts), INTENT(inout)          :: biomass
    ! density of individuals (1/m**2)
    REAl(stnd), DIMENSION(npts,npft), INTENT(inout)                 :: ind
    ! metabolic and structural litter, natural and agricultural,
    !   above and below ground (gC/m**2)
    REAL(stnd), DIMENSION(npts,nlitt,nvegtypes,nlevs), INTENT(inout):: litter
    ! dead leaves on ground, per PFT, metabolic and structural,
    !   in gC/(m**2 of nat/agri ground)
    REAL(stnd), DIMENSION(npts,npft,nlitt), INTENT(inout)           :: dead_leaves
    ! conversion of biomass to litter (gC/(m**2 of average nat/agri ground)) / day
    REAL(stnd), DIMENSION(npts,npft,nparts), INTENT(inout)          :: bm_to_litter
    ! black carbon on the ground (gC/(m**2 of total ground))
    REAL(stnd), DIMENSION(npts), INTENT(inout)                      :: black_carbon

    ! 0.3 output

    ! carbon emitted into the atmosphere by fire (living and dead biomass)
    ! (in gC/m**2/day)
    REAL(stnd), DIMENSION(npts), INTENT(out)                        :: co2_fire

    ! 0.4 local

    ! Time scale for memory of the fire index (days). Validated for one year in the DGVM.
    REAL(stnd), PARAMETER                                           :: tau_fire = 30.  ! GKtest
    ! Critical litter quantity for fire
    REAL(stnd), PARAMETER                                           :: litter_crit = 200.
    ! fire perturbation
    REAL(stnd), DIMENSION(npts)                                     :: fire_disturb
    ! what fraction of the plants is burned each day?
    REAL(stnd), DIMENSION(npts,npft)                                :: firedeath 
    ! What fraction of a burned plant compartment goes into the atmosphere
    !   (rest into litter)
    REAL(stnd), SAVE, DIMENSION(nparts)                             :: co2frac
    ! Moisture limit (critical moisture limit)
    REAL(stnd), DIMENSION(npts)                                     :: moistlimit
    ! total litter above the ground for a vegetation type (gC/m**2)
    REAL(stnd), DIMENSION(npts)                                     :: litter_above
    ! daily fire index
    REAL(stnd), DIMENSION(npts,nvegtypes)                           :: fireindex_daily
    ! fire extent, on natural and agricultural ground
    REAL(stnd), DIMENSION(npts, nvegtypes)                          :: firefrac
    ! residual fraction of exposed structural litter, depending on lignin fraction
    REAL(stnd), DIMENSION(npts)                                     :: struc_residual
    ! residue, i.e. exposed carbon - volatilized carbon ( gC/(m**2 of nat/agri ground))
    REAL(stnd), DIMENSION(npts)                                     :: residue
    ! fraction of residue transformed into black carbon
    REAL(stnd), DIMENSION(npts)                                     :: bcfrac
    ! fraction of total space that is natural/agricultural. Allows to transform from
    ! 1/(m**2 of nat/agri ground) to 1/(m**2 of total ground)
    REAL(stnd), DIMENSION(npts,nvegtypes)                           :: fracspace_vegtype
    ! intermediate variable
    REAL(stnd), DIMENSION(npts)                                     :: x
    ! annual fire fraction
    REAL(stnd), DIMENSION(npts)                                     :: aff
    ! are we talking about natural or agricultural ground?
    INTEGER(long)                                                   :: ivegtype
    ! index
    INTEGER(long)                                                   :: j,k,m

    ! =========================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering fire'

    !
    ! 1 Initializations
    !

    IF ( firstcall ) THEN

      !
      ! 1.1 What fraction of the plant compartment, if burned, is transformed into CO2
      !

      co2frac(ileaf) = .95
      co2frac(isapabove) = .95
      co2frac(isapbelow) = 0.
      co2frac(iheartabove) = 0.3
      co2frac(iheartbelow) = 0.
      co2frac(iroot) = 0.
      co2frac(ifruit) = .95
      co2frac(icarbres) = 0.95

      !
      ! 1.2 messages
      !

      WRITE(numout,*) 'fire:'

      WRITE(numout,*) '   > temporal inertia of fire probability (d): ',tau_fire

      WRITE(numout,*) '   > fraction of burned biomass that becomes CO2:'
      WRITE(numout,*) '     leaves: ', co2frac(ileaf)
      WRITE(numout,*) '     sap above ground: ', co2frac(isapabove)
      WRITE(numout,*) '     sap below ground: ', co2frac(isapbelow)
      WRITE(numout,*) '     heartwood above ground: ', co2frac(iheartabove)
      WRITE(numout,*) '     heartwood below ground: ', co2frac(iheartbelow)
      WRITE(numout,*) '     roots: ', co2frac(iroot)
      WRITE(numout,*) '     fruits: ', co2frac(ifruit)
      WRITE(numout,*) '     carbohydrate reserve: ', co2frac(icarbres)

      WRITE(numout,*) '   > critical litter quantity (gC/m**2): ',litter_crit
      WRITE(numout,*) '   > We calculate a fire probability on agricultural ground, but'
      WRITE(numout,*) '       the effective fire fraction is zero.'

      firstcall = .FALSE.

    ENDIF

    !
    ! 1.4 Initialize output
    !

    co2_fire(:) = 0.0
    firedeath(:,:) = 0.0

    !
    ! 1.5 fraction of total space reserved for vegetation type
    !

    fracspace_vegtype(:,inat) = space_nat(:)
    fracspace_vegtype(:,iagri) = 1. - space_nat(:)

    !
    ! 2 Determine fire probability. We calculate this probability (and long-term litter)
    !   also for agricultural ground, but the fire fraction on agricultural ground is set
    !   to 0 for the moment.
    !


    DO ivegtype = 1, nvegtypes

      ! total litter above the ground, for the vegetation type we are talking about

      litter_above(:) = litter(:,imetabolic,ivegtype,iabove) + &
                        litter(:,istructural,ivegtype,iabove)

      !
      ! 2.1 calculate moisture limit. If it stays 0, this means that there is no litter
      !     on the ground, and this means that there can be no fore.
      !     Sum over different litter parts from the various PFTs, taking into account the 
      !       litter flamability which is a function of the PFT. 
      !     Difference to Stephen Sitch's approach: Daily litter, not annual mean.
      !     Reason: 1. seems more reasonable.
      !             2. easier to implement (otherwise, would need moisture limit
      !                from previous year)
      !

      moistlimit(:) = 0.

      DO j = 1, npft

        ! If we are on natural ground, only take natural PFTs, and vice versa

        IF ( ( ( ivegtype .EQ. inat ) .AND. natural(j) ) .OR. &
             ( ( ivegtype .EQ. iagri ) .AND. .NOT. natural(j) ) ) THEN

          ! fire only when above feezing point and when there is litter
          !   (structural or metabolic) above the ground.
          ! Loop over grid points is the innermost because of vectorization. 
          !   (Makes the code unreadable.)

          WHERE ( ( t2m_daily(:) .GT. ZeroCelsius ) .AND. &
                  ( litter_above(:) .GT. min_stomate  )        )

            moistlimit(:) = moistlimit(:) + &
              ( litterpart(:,j,imetabolic)*litter(:,imetabolic,ivegtype,iabove) + &
                litterpart(:,j,istructural)*litter(:,istructural,ivegtype,iabove)  ) / &
              litter_above(:) * flam(j)

          ENDWHERE

        ENDIF     ! PFT and vegetation type coherent?

      ENDDO       ! PFT

      !
      ! 2.2 daily fire index.
      !

      WHERE ( moistlimit(:) .GT. 0.0 )

        ! is a function of litter humidity. Very sensible to STOMATE's time step as
        ! with larger dt, one misses dry days with very high fireindex ( strongly
        ! nonlinear: exp(-x^2)! )

        x(:) = litterhum_daily(:)/moistlimit(:)
        fireindex_daily(:,ivegtype) = EXP( - pi * x(:) * x(:) )

      ELSEWHERE

        fireindex_daily(:,ivegtype) = 0.0

      ENDWHERE

      !
      ! 2.3 increase long-term fire index (mean probability)
      !

      fireindex(:,ivegtype) = &
        ( ( tau_fire - dt ) * fireindex(:,ivegtype) + &
          ( dt )      * fireindex_daily(:,ivegtype)    ) / tau_fire

      !
      ! 2.4 litter influences fire intensity.
      !     We use longer-term litter to be consistent with the fire index.
      !

      firelitter(:,ivegtype) = &
        ( ( tau_fire-dt ) * firelitter(:,ivegtype) + dt * litter_above(:) ) / tau_fire

    ENDDO

    !
    ! 3 Calculate fire fraction from litter and fireindex (i.e. basically drought)
    !   We assume that agricultural space is separated from natural space so that
    !   the fire occurence on natural ground does not depend on the litter
    !   on agricultural ground.
    !

    !
    ! 3.1 natural ground
    !

    !     This formulation has been developped for annual fire indices!
    !     original form: firefrac(i) = fireindex(i) * EXP( f(fireindex(i)) )
    !     Transform into daily fire fraction.

    ! annual fire fraction

    aff(:) = firefrac_func (npts, fireindex(:,inat))

    ! transform from annual fraction to daily fraction. 
    ! annual fire fraction = 1. - (fraction of tree that survives each day) ** 365 =
    !                      = 1. - ( 1. - daily fire fraction )**365
    ! Thus, daily fire fraction = 1. - ( 1. - annual fire fraction )**(1/365)
    ! If annual firefrac<<1, then firefrac_daily = firefrac * dt/one_year
    ! This approximation avoids numerical problems.
  
    WHERE ( aff(:) .GT. 0.1 )
      firefrac(:,inat) = 1. - ( 1. - aff(:) ) ** (dt/one_year)
    ELSEWHERE
      firefrac(:,inat) = aff(:) * dt/one_year
    ENDWHERE

    ! No fire if litter is below critical value

    WHERE ( firelitter(:,inat) .LT. litter_crit )
      firefrac(:,inat) = 0.0
    ENDWHERE

    ! However, there is a minimum fire extent

    firefrac(:,inat) = MAX( 0.001_stnd * dt/one_year, firefrac(:,inat) )

    !
    ! 3.2 agricultural ground: no fire for the moment
    !

    firefrac(:,iagri) = 0.0

    !
    ! 4 Determine fire impact: calculate fire disturbance for each PFT
    !

    DO j = 1, npft

      !
      ! 4.1 are we talking about a natural or an agricultural PFT?
      !

      IF ( natural(j) ) THEN
        ivegtype = inat
      ELSE
        ivegtype = iagri
      ENDIF

      !
      ! 4.2 fire disturbance
      !

      IF ( tree(j) ) THEN

        ! 4.2.1 Trees: always disturbed

        fire_disturb(:) = ( 1. - resist(j) ) * firefrac(:,ivegtype)

      ELSE

        ! 4.2.2 Grasses are not disturbed if they are not in their growing season

        WHERE ( biomass(:,j,ileaf) .GT. min_stomate )

          fire_disturb(:) = ( 1. - resist(j) ) * firefrac(:,ivegtype)

        ELSEWHERE

          fire_disturb(:) = 0.0

        ENDWHERE

      ENDIF

      !
      ! 4.3 litter and co2 created through fire on living biomass
      !

      ! biomass can go into litter or atmosphere, depending on what plant compartment
      ! we are talking about.

      DO k = 1, nparts

        ! grass roots and reserve survive.

        IF ( .NOT. ( ( .NOT. tree(j) ) .AND. ( ( k.EQ.iroot ) .OR. ( k.EQ.icarbres ) ) ) ) THEN

          ! 4.3.1 A fraction goes directly into the atmosphere.
          !       CO2 flux in gC/m**2 of total ground/day.

          co2_fire(:) = co2_fire(:) + biomass(:,j,k) * fire_disturb(:) * co2frac(k) * &
                        fracspace_vegtype(:,ivegtype) / dt

          ! 4.3.2 Determine the residue, in gC/m**2 of natural/agricultural ground.

          residue(:) = biomass(:,j,k) * fire_disturb(:) * ( 1. - co2frac(k) )

          ! 4.3.2.1 determine fraction of black carbon. Only for plant parts above the
          !         ground, i.e. when co2_frac > 0.
          !         A small part of the residue, which can be expressed as a function of
          !         the fraction of volatilized carbon (assimilated to co2frac here),
          !         becomes black carbon, and thus withdrawn from the soil carbon cycle (added
          !         to the "geologic carbon cycle we don't care about here).
          !         [Kuhlbusch et al. JGR 101, 23651-23665, 1996; Kuhlbusch & Crutzen, GBC 9,
          !          491-501, 1995].
  
          IF ( co2frac(k) .GT. 0.0 ) THEN
            bcfrac(:) = .3 / ( 1.3 ** ( 88.2 - 100.*co2frac(k) ) + 1. )
          ELSE
            bcfrac(:) = 0.0
          ENDIF
  
          ! 4.3.2.2 Add this fraction of the residue to the black carbon "reservoir", in
          !         gC/(m**2 of total ground).
  
          black_carbon(:) = &
             black_carbon(:) + bcfrac(:) * residue(:) * fracspace_vegtype(:,ivegtype)
  
          ! 4.3.2.3 The rest (largest part) of the residue becomes litter.
  
          bm_to_litter(:,j,k) = bm_to_litter(:,j,k) + residue(:) * ( 1 - bcfrac(:) )

        ENDIF  ! not for grass roots
  
      ENDDO

      !
      ! 4.4 new vegetation characteristics
      !

      ! 4.4.1 decrease biomass per m**2 of natural(/agricultural) ground
      !       except for grass roots.

      DO k = 1, nparts

        IF ( .NOT. ( ( .NOT. tree(j) ) .AND. ( ( k.EQ.iroot ) .OR. ( k.EQ.icarbres) ) ) ) THEN

          biomass(:,j,k) = ( 1. - fire_disturb(:) ) * biomass(:,j,k)

        ENDIF

      ENDDO

      ! 4.4.2 If vegetation is dynamic, then decrease the density of tree
      !       individuals.

      IF ( control%ok_dgvm .AND. tree(j) ) THEN

        ! fraction of plants that dies each day.
        ! exact formulation: 1. - (1.-fire_disturb(:)) ** (1./dt)
        firedeath(:,j) = fire_disturb(:) / dt

        ind(:,j) = ( 1. - fire_disturb(:) ) * ind(:,j)

      ENDIF

    ENDDO      ! loop over PFTs

    !
    ! 5 A fraction of the (natural) litter is burned by the fire
    !

    DO ivegtype = 1,nvegtypes

      !
      ! 5.1 exposed metabolic litter burns totally and goes directly into the atmosphere as
      !     CO2.
      !

      ! 5.1.1 CO2 flux, in gC/(m**2 of total ground)/day.

      co2_fire(:) = co2_fire(:) + litter(:,imetabolic,ivegtype,iabove) * &
                                  firefrac(:,ivegtype) * fracspace_vegtype(:,ivegtype) / dt

      ! 5.1.2 decrease metabolic litter

      litter(:,imetabolic,ivegtype,iabove) = litter(:,imetabolic,ivegtype,iabove) * &
                                             ( 1. - firefrac(:,ivegtype) )

      !
      ! 5.2 exposed structural litter is not totally transformed into CO2.
      !

      ! 5.2.1 Fraction of exposed structural litter that does not
      !       burn totally should depend on lignin content (lignin_struc). VERY TENTATIVE!

      struc_residual(:) = 0.5 * lignin_struc(:,ivegtype,iabove)

      ! 5.2.2 CO2 flux, in gC/(m**2 of total ground)/day.

      co2_fire(:) = co2_fire(:) + &
                    litter(:,istructural,ivegtype,iabove) * firefrac(:,ivegtype) * &
                      ( 1. - struc_residual(:) ) * fracspace_vegtype(:,ivegtype) / dt

      ! 5.2.3 determine residue (litter that undergoes fire, but is not transformed
      !       into CO2)

      residue(:) = litter(:,istructural,ivegtype,iabove) * firefrac(:,ivegtype) * &
                   struc_residual(:)

      ! 5.2.4 determine fraction of black carbon in the residue.
      !       depends on volatilized fraction of carbon (see 4.3.2.1)

      bcfrac(:) = .3 / ( 1.3 ** ( 88.2 - 100.*(1.-struc_residual(:)) ) + 1. )

      ! 5.2.5 Add this fraction of the residue to the black carbon "reservoir", in
      !       gC/(m**2 of total ground).

      black_carbon(:) = &
         black_carbon(:) + bcfrac(:) * residue(:) * fracspace_vegtype(:,ivegtype)

      ! 5.2.6 The rest (largest part) of the residue remains litter. Remaining litter
      !       is the sum of this and of the litter which has not undergone a fire.

      litter(:,istructural,ivegtype,iabove) = &
             litter(:,istructural,ivegtype,iabove) * ( 1. - firefrac(:,ivegtype) ) + &
             residue(:) * ( 1. - bcfrac(:) )

    ENDDO  ! natural/agricultural ground

    !
    ! 5.3 diagnose fraction of leaves burned.
    !     exposed leaves are burned entirely, even their structural part
    !

    DO j = 1, npft

      IF ( natural(j) ) THEN
        m = inat
      ELSE
        m = iagri
      ENDIF

      DO k = 1, nlitt
        dead_leaves(:,j,k) = dead_leaves(:,j,k) * ( 1. - firefrac(:,m) )
      ENDDO

    ENDDO

    !
    ! 6 history
    !

    ! output in 1/day
    firefrac = firefrac / dt

    CALL histwrite (hist_id_stomate, 'FIREFRAC_NAT', itime, &
                    firefrac(:,inat), npts, hori_index)
    CALL histwrite (hist_id_stomate, 'FIREFRAC_AGRI', itime, &
                    firefrac(:,iagri), npts, hori_index)
    CALL histwrite (hist_id_stomate, 'FIREDEATH', itime, &
                    firedeath(:,:), npts*npft, horipft_index)

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving fire'

  END SUBROUTINE fire

  FUNCTION firefrac_func (npts, x) RESULT (firefrac_result)

    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                             :: npts
    ! fire index
    REAL(stnd), DIMENSION(npts), INTENT(in)                         :: x

    ! 0.2 result

    ! fire fraction
    REAL(stnd), DIMENSION(npts)                                     :: firefrac_result

    ! 0.3 local

    ! intermediate variable
    REAL(stnd), DIMENSION(npts)                                     :: xm1

    xm1(:) = x(:) - 1.

    firefrac_result(:) = &
      x(:) * EXP( xm1(:) / ( -.13*xm1(:)*xm1(:)*xm1(:) + .6*xm1(:)*xm1(:) + .8*xm1(:) + .45 ) )

  END FUNCTION firefrac_func

END MODULE lpj_fire
